@@ -281,34 +281,80 @@ def _diff1(a, vec, time, i = 0, t = np.nan):
     return res, vec, i, t
 
 
+# @loop_all
+# @pd2np
+# @compiled
+# def _buffer(a, band, unit = 0.0, pos = 0, rounding_band = 0):
+#     """
+    
+#     Handles buffering when the rounding into units may be significant cost
+    
+    
+#     """
+#     res = np.full(a.shape, np.nan)
+#     b = floor = rounding_band * unit
+#     if np.isnan(pos):
+#         pos = 0.0
+#     for i in range(a.shape[0]):
+#         if not np.isnan(a[i]):
+#             if not np.isnan(band[i]): ## we forward fill band
+#                 b = max(band[i], floor)
+#             if pos < a[i] - b:
+#                 pos = a[i] - b
+#             elif pos > a[i] + b:
+#                 pos = a[i] + b
+#             res[i] = pos
+#     if unit > 0:
+#         pos = np.round(pos / unit) * unit
+#     return res, pos
+
+
 @loop_all
 @pd2np
 @compiled
-def _buffer(a, band, unit = 0.0, pos = 0, rounding_band = 0.5):
+def _buffer(a, band, unit = 0.0, pos = 0, rounding_band = 0):
     """
-    
-    Handles buffering when the rounding into units may be significant cost
-    
-    
+    >>> from pyg import * 
+    >>> a = pd.Series(cumsum(np.random.normal(0,1,1000)), drange(-999))
+    >>> sig = ewmacd(a, 4, 12, vol = 18)
+    >>> sig.plot()
+    >>> ts = buffer(sig, band = 0.1, unit = 1, rounding_band = 0)
+    >>> df_concat([a,ts])[dt(-100):].plot()
     """
     res = np.full(a.shape, np.nan)
-    b = floor = rounding_band * unit
+    rbu = rounding_band * unit
     if np.isnan(pos):
         pos = 0.0
     for i in range(a.shape[0]):
         if not np.isnan(a[i]):
-            if not np.isnan(band[i]): ## we forward fill band
-                b = np.max(band[i], floor)
+            b = band[i]
             if pos < a[i] - b:
-                pos = a[i] - b
+                aim = a[i] - b
+                if unit > 0:
+                    aim = np.round(aim / unit) * unit
+                    if aim < a[i] - b and aim + unit < a[i] + b: # don't undershoot
+                        pos = aim + unit
+                    elif aim > a[i] and rbu > b and a[i] - rbu > pos: # don't overshoot
+                        aim = a[i] - rbu
+                        pos = np.round(aim / unit) * unit
+                    else:
+                        pos = aim
+                else:
+                    pos = aim
             elif pos > a[i] + b:
-                pos = a[i] + b
+                aim = a[i] + b
+                if unit > 0:
+                    aim = np.round(aim / unit) * unit
+                    if aim > a[i] + b and aim - unit > a[i] - b:
+                        pos = aim - unit
+                    elif aim < a[i] and rbu > b and a[i] + rbu < pos: # don't overshoot
+                        aim = a[i] + rbu
+                        pos = np.round(aim / unit) * unit
+                    else:
+                        pos = aim
             res[i] = pos
-    if unit > 0:
-        pos = np.round(pos / unit) * unit
     return res, pos
 
-
 @loop_all
 def _tdiff(a, n, vec, i, time = None, t = None):
     time = clock(a, time, t)
@@ -801,7 +847,7 @@ def diff_(a, n=1, time = None, axis = 0, data = None, instate = None):
 
 diff_.output = ['data', 'state']
 
-def buffer_(a, band, unit = 0.0, rounding_band = 0.5, data = None, instate = None, rms = None):
+def buffer_(a, band, unit = 0.0, rounding_band = 0., data = None, instate = None, rms = None):
     if is_num(instate):
         instate = Dict(pos = instate)
     elif instate is None:
@@ -815,7 +861,7 @@ def buffer_(a, band, unit = 0.0, rounding_band = 0.5, data = None, instate = Non
         
 buffer_.output = ['data', 'state']
 
-def buffer(a, band, unit = 0.0, rounding_band = 0.5, data = None, state = None, rms = None):
+def buffer(a, band, unit = 0.0, rounding_band = 0., data = None, state = None, rms = None):
     """
     buffer performs two functions:
         - ensures the result is stated in 'units' so if unit == 1, output is integers
